#!/usr/bin/env python
#
#    imap2mbox.py v1.5.0
#
#    Copyright (C) 2008/2009 Michele <o-zone@zerozone.it> Pinassi
#
#    This software is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This software is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    1/9/2008 - First public release
#    29/10/2008 (v1.1.0) Added patches from Gregory Boyce <gboyce@badbelly.com> (OptionParser, SSL, read-only, skip not-matched) and some small fixes by me
#    18/11/2008 (v1.2.0) New patches from Gregory Boyce <gboyce@badbelly.com> that adds support for Exchange. Added compatibility with Python 2.3
#    13/03/2009 (v1.2.1) Added fixes on password dialog from Peter Huewe <PeterHuewe@gmx.de>
#    26/03/2009 (v1.3.0) Added new options FromDate, ToDate and MarkDelete
#    21/04/2009 (v1.3.1) Some minor fixes
#    24/08/2009 (v1.4.0) Added (beta) HTML export of mailbox
#    01/09/2009 (v1.4.1) Fixes GMail issue and add -v option
#    14/10/2009	(v1.5.0) imap2mbox converted to a class to let users build GUI and others...

import getpass, imaplib, os, email, sys, getopt, re
from optparse import OptionParser
from datetime import tzinfo, timedelta, datetime
from email.utils import getaddresses
import mimetypes

if sys.version_info[0] >= 2 and sys.version_info[1] >= 5:
    # do stuff for Python 2.5
    from email.utils import unquote
else:
    from rfc822 import unquote

from fnmatch import *

class progressBar:
	def __init__(self, minValue = 0, maxValue = 10, totalWidth=12):
		self.progBar = "[]"   # This holds the progress bar string
		self.min = minValue
		self.max = maxValue
		self.span = maxValue - minValue
		self.width = totalWidth
		self.amount = 0       # When amount == max, we are 100% done 
		self.updateAmount(0)  # Build progress bar string

	def updateAmount(self, newAmount = 0):
		if newAmount < self.min: newAmount = self.min
		if newAmount > self.max: newAmount = self.max
		self.amount = newAmount

		# Figure out the new percent done, round to an integer
		diffFromMin = float(self.amount - self.min)
		percentDone = (diffFromMin / float(self.span)) * 100.0
		percentDone = round(percentDone)
		percentDone = int(percentDone)

		# Figure out how many hash bars the percentage should be
		allFull = self.width - 2
		numHashes = (percentDone / 100.0) * allFull
		numHashes = int(round(numHashes))

		# build a progress bar with hashes and spaces
		self.progBar = "[" + '#'*numHashes + ' '*(allFull-numHashes) + "]"

		# figure out where to put the percentage, roughly centered
		percentPlace = (len(self.progBar) / 2) - len(str(percentDone)) 
		percentString = str(percentDone) + "%"

		# slice the percentage into the bar
		self.progBar = self.progBar[0:percentPlace] + percentString + self.progBar[percentPlace+len(percentString):]

	def __str__(self):
		return str(self.progBar)

class imap2mbox:
    imapHost = ""
    imapUser = ""
    imapPass = ""
    imapFolder = "*"
    imapPort = 143
    imapOutput = "mbox"

    imapFromDate = datetime(1980,1,1)
    imapToDate = datetime.now()
    
    imapError = ""
    
    isSsl = False
    isVerbose = False
    isReadonly = False
    isTest = False
    isMarkDelete = False

    messageSaved = 0
    messageSkipped = 0

    htmlMexList = ""

    # Parse list
    def parseList(self,listEntry):
	parser = re.compile('(\(.*\)) (".*") (.*)')
        return parser.match(listEntry).groups() 

    def doLogin(self):
	try:
	    # If SSL...
	    if self.isSsl:
		if self.isVerbose:
		    print "VERBOSE: Connecting to %s:%s using SSL..."%(self.imapHost,self.imapPort)
		self.imap4 = imaplib.IMAP4_SSL(self.imapHost,self.imapPort)
	    # else normal...
	    else:
		if self.isVerbose:
		    print "VERBOSE: Connecting to %s:%s..."%(self.imapHost,self.imapPort)
		self.imap4 = imaplib.IMAP4(self.imapHost,self.imapPort)
	    # Let's go !
	    if self.imap4:
		if self.isVerbose:
		    print "VERBOSE: Logging to %s:%s..."%(self.imapUser,self.imapPass)

		self.imap4.login(self.imapUser, self.imapPass)
		if self.isReadonly:
		    self.imap4.select(readonly=True)
		else:
		    self.imap4.select()
	    else:
		self.imapError = "Login error"
		return False	
	except Exception,e:
	    self.imapError = "ERROR: IMAP4 error %r\n"%(e)
	    return False
	    
	return self.imap4

    def doLogout(self):
	self.imap4.close()
	self.imap4.logout()

    def printSummary(self):
        ret, mboxes = self.imap4.list()
	for mbox in mboxes:
    	    hasChild, dot, mbName = self.parseList(mbox)
	    mbName = unquote(mbName)
	    try:
		if self.isReadonly:
		    ret, numMex = self.imap4.select(mbName,readonly=True)
		else:
		    ret, numMex = self.imap4.select(mbName)
		print "%s has %d messages"%(mbName,int(numMex[0]))
	    except Exception,e:
		self.imapError = "ERROR: %r\n"%e

    def exportHtml(self,mbName,mexNum,msgData,msgDate):
	# Azzera variabili
	htmlAttachs = ""
	htmlBody = ""			    
	ext = ""
	# Decodifica mail in HTML e salva eventuali allegati
	mboxFd = os.open(os.path.join(mbName,"message%04d.html"%(int(mexNum))),os.O_WRONLY | os.O_CREAT)
	counter = 1
	# Salva allegati
	for part in msgData.walk():
    	    # multipart/* are just containers
    	    if part.get_content_maintype() == 'multipart':
        	continue
    	    # Applications should really sanitize the given filename so that an
    	    # email message can't be used to overwrite important files
	    filename = part.get_filename()
    	    if not filename:
    		if part.get_content_type() == "text/plain":
    		    htmlBody += part.get_payload(decode=True)
    		    continue
    		else:
    		    ext = mimetypes.guess_extension(part.get_content_type())
    		    
    	    if not ext:
                # Use a generic bag-of-bits extension
                ext = '.bin'
    		filename = 'part-%03d%s' % (counter, ext)

    	    counter += 1
    	    try:
    		os.mkdir(os.path.join(mbName,"message%04d"%(int(mexNum))))
    	    except:
    		pass
    	
    	    try:    
    		attPayload = part.get_payload(decode=True)
    		if attPayload == None:
    		    continue
    		else:
    		    fp = open(os.path.join(mbName,"message%04d"%(int(mexNum)),filename), 'wb')
    		    fp.write(attPayload)
    		    fp.close()
    		    htmlAttachs += "<li>[%d]<a href='%s'>%s</a></li>\n"%(counter,os.path.join(mbName,"message%04d"%(int(mexNum)), filename),filename)
    	    except Exception,e:
    		print "ERROR: Failed saving %s because: %r"%("message%04d"%(int(mexNum)),e)
    				    
        # Build mail header
        subject = msgData.get_all('subject', [])
        tos = msgData.get_all('to', [])
	froms = msgData.get_all('from', [])
	ccs = msgData.get_all('cc', [])
	messids = msgData.get_all('message-id', [])
    	
    	htmlHeadField = "<b>To: %s</b><br>\n"%",".join(tos)
    	htmlHeadField += "<b>From: %s</b><br>\n"%",".join(froms)
    	htmlHeadField = "<b>Subject: %s</b><br>\n"%(subject[0])
    	htmlHeadField += "<b>CCs: %s</b><br>\n"%",".join(ccs)
    	htmlHeadField += "<b>Date: %s</b><br>\n"%datetime(msgDate[0],msgDate[1],msgDate[2],msgDate[3],msgDate[4],msgDate[5])
	htmlHeadField += "<hr>\n"

    	htmlHeader = "<html><!-- MAILID:%s --><head><title>%s</title></head>\n"%(messids[0],subject[0])
    	htmlFooter = "<br clear='all'><p align='center'>build by imap2mbox (C) 2008-2009 Michele Pinassi</p>\n";
    			        			
    	# Now write file
    			    
	os.write(mboxFd,str(htmlHeader))
    	os.write(mboxFd,"<body>\n")
    	os.write(mboxFd,str(htmlHeadField))
    	os.write(mboxFd,"<pre>"+str(htmlBody)+"</pre>")
    	os.write(mboxFd,"<ul>"+str(htmlAttachs)+"</ul>")
    	os.write(mboxFd,str(htmlFooter))
    	os.write(mboxFd,"</body></html>\n")
	os.close(mboxFd)
	    
	# Update global mails list
	self.htmlMexList += "<!-- MAILID:%s --><li>[%d] <a href='%s'>%s</a> from %s</li>\n"%(messids[0],int(mexNum),os.path.join(mbName,"message%04d.html"%(int(mexNum))),subject[0],",".join(froms))
    
    def doSave(self):
	if self.isVerbose:
	    print "VERBOSE: Saving all messages from %s to %s"%(self.imapFromDate,self.imapToDate)    
	try:
	    os.mkdir(self.imapHost)
	except:
	    pass
	os.chdir(self.imapHost)
	try:
	    os.mkdir(self.imapUser)
	except:
	    pass
	os.chdir(self.imapUser)
	# Inizia ad importare la casella postale
	ret, mboxes = self.imap4.list()
	for mbox in mboxes:
	    hasChild, dot, mbName = self.parseList(mbox)
	    mbName = unquote(mbName)
	    if self.isReadonly:
		ret, numMex = self.imap4.select(mbName,readonly=True)
	    else:
		ret, numMex = self.imap4.select(mbName)

	    if self.isVerbose:
		print "VERBOSE: %s (%r) has %r messages. Saving in %s"%(mbName,ret,numMex[0],self.imapOutput)
		
	    if ret == 'OK' and fnmatch(mbName, self.imapFolder):	    
		if int(numMex[0]) > 0:
        	    mbName = re.sub("/",".",mbName)
		    if os.path.isfile("%s.mbox"%(mbName)):
			print "%s.mbox exists. Skipping !"%(mbName)
		    else:
			if self.imapOutput == "html":
			    # Se salva in HTML crea directory con nome mbox...
			    try:
				os.mkdir(mbName)
			    except:
				print "WARNING: Directory %s already exists !"%(mbName)
			    htmlMexList = "<html><head><title>Messages into '%s'</title></head><body>\n<!-- IMAP2MBOX:START -->\n<ul>"%(mbName);
			else: 
			    # ...altrimenti crea file MBOX !
			    mboxFd = os.open("%s.mbox"%(mbName),os.O_WRONLY | os.O_CREAT)
			    
			numMex = int(numMex[0])    
			if self.isVerbose:
			    print "VERBOSE: Running on %s (%d messages):"%(mbName,numMex)
			ret, data = self.imap4.search(None, 'ALL')
			if self.isVerbose:
			    pBar = progressBar(0,numMex,20)
			    
			for num in data[0].split():
			    # Preleva messaggio
			    ret, data = self.imap4.fetch(num, '(RFC822)')
    			    msgData = email.message_from_string(data[0][1])    	    
    			    msgDate = email.Utils.parsedate(msgData.get('Date'))
    			    try:
    				tempFromDate = datetime(msgDate[0],msgDate[1],msgDate[2])
    			    except Exception,e:
    				print "ERROR: Date error %s"%e
    				self.messageSkipped = self.messageSkipped + 1
    			
    			    if(tempFromDate == None):
    				print "ERROR: Date error %s"%msgDate
    				self.messageSkipped = self.messageSkipped + 1
    			    elif(self.imapFromDate < tempFromDate < self.imapToDate):
		    		# Scrivi MESSAGGIO
		    		if self.imapOutput == "html":
				    self.exportHtml(mbName,num,msgData,msgDate)
		    		else:
				    os.write(mboxFd,str(msgData))
				    os.write(mboxFd,"\n")
					
				self.messageSaved = self.messageSaved + 1
				if self.isMarkDelete:
				    self.imap4.store(num, '+FLAGS', '\\Deleted')			
				else:
			    	    self.messageSkipped = self.messageSkipped + 1

			    if self.isVerbose:
				print pBar, "\r",
				pBar.updateAmount(int(num))
				sys.stdout.flush()
			
			# Finish !
			if self.isVerbose:
	    		    print pBar, "\r",
			    pBar.updateAmount(numMex)
			    sys.stdout.flush()
			    print "\n"
			
			if self.imapOutput == "html":
			    # Scrivi elenco mails
			    self.htmlMexList += "</ul>\n<!-- IMAP2MBOX:END -->\n</body></html>";
			    fp = os.open(mbName+".html", os.O_WRONLY | os.O_CREAT)
			    os.write(fp,self.htmlMexList)
			    os.close(fp)
			else:
			    # Chiudi file mbox
			    os.close(mboxFd)

			if self.isVerbose:			    
			    print "=== mbox %s have %d message saved and %d skipped ==="%(mbName,self.messageSaved,self.messageSkipped)
			    
	    else:
	    	if self.isVerbose:
		    if ret == 'NO':
			print "INFO: Skip invalid mbox %s"%(mbName)	    
		    else:
			print "INFO: Skip empty mbox %s"%(mbName)
	
	        
if __name__ == '__main__':
    # Parse commandline options
    parser = OptionParser()
    parser.add_option("-m", "--mailsrv", default=None,help="Mailserver to connect to")
    parser.add_option("-p", "--port", type="int",help="Port to connect to")
    parser.add_option("-u", "--username", default=getpass.getuser(), help="Username (defaults to current user)")
    parser.add_option("-w", "--password", default=None, help="Specify Password (less secure)")
    parser.add_option("-f", "--folder", default="*", help="Select folders to download (wildcards supported)")
    parser.add_option("-s", "--ssl", action="store_true", help="Enable SSL")		  
    parser.add_option("-v", "--verbose", action="store_true", help="Be verbose (debug)")		  
    parser.add_option("-r", "--readonly", action="store_true", help="Do not mark messages as read")		  
    parser.add_option("-t", "--test", action="store_true", help="Test Mode")
    parser.add_option("-o", "--output", default="mbox", help="How to save mails ? mbox (default) or html (save text and mime attachments)")
    parser.add_option("", "--fromdate", default=None, help="Start date (format: YYYY-MM-DD)")
    parser.add_option("", "--todate", default=None, help="To date (format: YYYY-MM-DD)")
    parser.add_option("", "--markdelete", action="store_true", help="Mark as deleted all saved messages")		  

    (options, args) = parser.parse_args()

    if options.mailsrv == None:
	parser.error("Argument 'mailsrv' missing.")

    if options.password == None:
	options.password = getpass.getpass()
    
    i2m = imap2mbox()
    i2m.imapHost = options.mailsrv
    if options.port:
	i2m.imapPort = options.port

    i2m.imapUser = options.username	
    i2m.imapPass = options.password

    if options.fromdate:
	userDate = options.fromdate.split('-')
	i2m.imapFromDate = datetime(int(userDate[0]),int(userDate[1]),int(userDate[2]))

    if options.todate:
	userDate = options.todate.split('-')
	i2m.imapToDate = datetime(int(userDate[0]),int(userDate[1]),int(userDate[2]))

    if options.output:
	i2m.imapOutput = options.output

    if options.markdelete:
	i2m.isMarkDelete = True

    if options.ssl:
	i2m.isSsl = True

    if options.verbose:
	i2m.isVerbose = True

    if options.readonly:
	i2m.isReadonly = True

    if i2m.doLogin():
	if options.test:
	    i2m.printSummary()
	else:
	    i2m.doSave()	
	    
	i2m.doLogout()
    else:
	print i2m.imapError
    
    
    